use axum::{
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use serde_json::json;
use thiserror::Error;

#[derive(Error, Debug)]
pub enum AppError {
    #[error("Authentication required")]
    Unauthorized,

    #[error("Access denied")]
    AccessDenied,

    #[error("User not found")]
    UserNotFound,

    #[error("User already exists")]
    UserAlreadyExists,

    #[error("Invalid credentials")]
    InvalidCredentials,

    #[error("Invalid or expired token")]
    InvalidToken,

    #[error("File not found")]
    FileNotFound,

    #[error("File already exists")]
    FileAlreadyExists,

    #[error("Directory already exists")]
    DirectoryAlreadyExists,

    #[error("Not a directory")]
    NotADirectory,

    #[error("Storage quota exceeded")]
    StorageQuotaExceeded,

    #[error("Path too deep")]
    PathTooDeep,

    #[error("Database error: {0}")]
    DatabaseError(#[from] sqlx::Error),

    #[error("Password hashing error: {0}")]
    PasswordHashingError(#[from] argon2::password_hash::Error),

    #[error("JWT error: {0}")]
    JwtError(#[from] jsonwebtoken::errors::Error),

    #[error("Encryption error: {0}")]
    EncryptionError(String),

    #[error("IO error: {0}")]
    IoError(#[from] std::io::Error),

    #[error("Invalid input: {0}")]
    ValidationError(String),

    #[error("Internal server error")]
    InternalServerError,
}

impl IntoResponse for AppError {
    fn into_response(self) -> Response {
        let (status, error_message) = match self {
            Self::Unauthorized => (StatusCode::UNAUTHORIZED, self.to_string()),
            Self::AccessDenied => (StatusCode::FORBIDDEN, self.to_string()),
            Self::UserNotFound | Self::FileNotFound => (StatusCode::NOT_FOUND, self.to_string()),
            Self::UserAlreadyExists | Self::FileAlreadyExists | Self::DirectoryAlreadyExists => {
                (StatusCode::CONFLICT, self.to_string())
            }
            Self::InvalidCredentials | Self::InvalidToken => {
                (StatusCode::UNAUTHORIZED, self.to_string())
            }
            Self::NotADirectory | Self::ValidationError(_) => {
                (StatusCode::BAD_REQUEST, self.to_string())
            }
            Self::StorageQuotaExceeded => {
                (StatusCode::PAYLOAD_TOO_LARGE, self.to_string())
            }
            Self::PathTooDeep => (StatusCode::BAD_REQUEST, self.to_string()),
            Self::DatabaseError(e) => {
                tracing::error!("Database error: {:?}", e);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
            Self::PasswordHashingError(e) => {
                tracing::error!("Password hashing error: {:?}", e);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
            Self::JwtError(e) => {
                tracing::error!("JWT error: {:?}", e);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
            Self::EncryptionError(e) => {
                tracing::error!("Encryption error: {}", e);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
            Self::IoError(e) => {
                tracing::error!("IO error: {:?}", e);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
            Self::InternalServerError => {
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error".to_string())
            }
        };

        let body = Json(json!({
            "error": error_message
        }));

        (status, body).into_response()
    }
}