use crate::error::AppError;
use serde::{Deserialize, Serialize};
use sqlx::PgPool;
use time::OffsetDateTime;
use uuid::Uuid;

#[derive(Debug, Serialize, Deserialize, sqlx::FromRow)]
pub struct Permission {
    pub id: Uuid,
    pub name: String,
    pub can_read: bool,
    pub can_write: bool,
    pub can_share: bool,
    pub created_at: OffsetDateTime,
    pub updated_at: OffsetDateTime,
}

impl Permission {
    pub async fn find_by_id(pool: &PgPool, id: Uuid) -> Result<Permission, AppError> {
        let permission = sqlx::query_as::<_, Permission>(
            r#"
            SELECT * FROM permissions WHERE id = $1
            "#,
        )
        .bind(id)
        .fetch_optional(pool)
        .await
        .map_err(AppError::from)?;

        permission.ok_or_else(|| AppError::NotFound("Permission not found".to_string()))
    }

    pub async fn find_by_name(pool: &PgPool, name: &str) -> Result<Permission, AppError> {
        let permission = sqlx::query_as::<_, Permission>(
            r#"
            SELECT * FROM permissions WHERE name = $1
            "#,
        )
        .bind(name)
        .fetch_optional(pool)
        .await
        .map_err(AppError::from)?;

        permission.ok_or_else(|| AppError::NotFound(format!("Permission '{}' not found", name)))
    }

    pub async fn list_all(pool: &PgPool) -> Result<Vec<Permission>, AppError> {
        let permissions = sqlx::query_as::<_, Permission>(
            r#"
            SELECT * FROM permissions
            "#,
        )
        .fetch_all(pool)
        .await
        .map_err(AppError::from)?;

        Ok(permissions)
    }

    pub async fn create_default_permissions(pool: &PgPool) -> Result<(), AppError> {
        // Check if permissions already exist
        let existing = Self::list_all(pool).await?;
        if !existing.is_empty() {
            return Ok(());
        }

        // Create default permission levels
        let permissions = [
            ("viewer", true, false, false),
            ("editor", true, true, false),
            ("admin", true, true, true),
        ];

        for (name, can_read, can_write, can_share) in permissions {
            sqlx::query(
                r#"
                INSERT INTO permissions (name, can_read, can_write, can_share)
                VALUES ($1, $2, $3, $4)
                ON CONFLICT (name) DO NOTHING
                "#,
            )
            .bind(name)
            .bind(can_read)
            .bind(can_write)
            .bind(can_share)
            .execute(pool)
            .await
            .map_err(AppError::from)?;
        }

        Ok(())
    }
}