use crate::error::AppError;
use crate::models::file::File;
use crate::models::permission::Permission;
use crate::models::user::User;
use serde::{Deserialize, Serialize};
use sqlx::PgPool;
use time::OffsetDateTime;
use uuid::Uuid;

#[derive(Debug, Serialize, Deserialize, sqlx::FromRow)]
pub struct Share {
    pub id: Uuid,
    pub file_id: Uuid,
    pub owner_id: Uuid,
    pub recipient_id: Option<Uuid>,
    pub permission_id: Uuid,
    pub access_key: String,
    pub expires_at: Option<OffsetDateTime>,
    pub created_at: OffsetDateTime,
    pub updated_at: OffsetDateTime,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct CreateShareDto {
    pub file_id: Uuid,
    pub recipient_id: Option<Uuid>,
    pub permission_id: Uuid,
    pub expires_at: Option<OffsetDateTime>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct ShareResponse {
    pub id: Uuid,
    pub file: File,
    pub owner: User,
    pub recipient: Option<User>,
    pub permission: Permission,
    pub access_key: String,
    pub expires_at: Option<OffsetDateTime>,
    pub created_at: OffsetDateTime,
    pub updated_at: OffsetDateTime,
}

impl Share {
    pub async fn create(
        pool: &PgPool,
        owner_id: Uuid,
        dto: CreateShareDto,
    ) -> Result<Share, AppError> {
        // Verify file exists and user has access to it, then create a random key for it
        let file = File::find_by_id(pool, dto.file_id).await?
        
        if file.owner_id != owner_id {
            return Err(AppError::AccessDenied("You can only share files you own".to_string()));
        }
        let access_key = Uuid::new_v4().to_string();

        let share = sqlx::query_as::<_, Share>(
            r#"
            INSERT INTO shares (file_id, owner_id, recipient_id, permission_id, access_key, expires_at)
            VALUES ($1, $2, $3, $4, $5, $6)
            RETURNING *
            "#,
        )
        .bind(dto.file_id)
        .bind(owner_id)
        .bind(dto.recipient_id)
        .bind(dto.permission_id)
        .bind(&access_key)
        .bind(dto.expires_at)
        .fetch_one(pool)
        .await
        .map_err(AppError::from)?;

        Ok(share)
    }

    pub async fn find_by_id(pool: &PgPool, id: Uuid) -> Result<Share, AppError> {
        let share = sqlx::query_as::<_, Share>(
            r#"
            SELECT * FROM shares WHERE id = $1
            "#,
        )
        .bind(id)
        .fetch_optional(pool)
        .await
        .map_err(AppError::from)?;

        share.ok_or_else(|| AppError::NotFound("Share not found".to_string()))
    }

    pub async fn find_by_access_key(pool: &PgPool, access_key: &str) -> Result<Share, AppError> {
        let share = sqlx::query_as::<_, Share>(
            r#"
            SELECT * FROM shares WHERE access_key = $1
            "#,
        )
        .bind(access_key)
        .fetch_optional(pool)
        .await
        .map_err(AppError::from)?;

        share.ok_or_else(|| AppError::NotFound("Share not found".to_string()))
    }

    pub async fn list_by_owner(pool: &PgPool, owner_id: Uuid) -> Result<Vec<Share>, AppError> {
        let shares = sqlx::query_as::<_, Share>(
            r#"
            SELECT * FROM shares WHERE owner_id = $1
            "#,
        )
        .bind(owner_id)
        .fetch_all(pool)
        .await
        .map_err(AppError::from)?;

        Ok(shares)
    }

    pub async fn list_by_recipient(pool: &PgPool, recipient_id: Uuid) -> Result<Vec<Share>, AppError> {
        let shares = sqlx::query_as::<_, Share>(
            r#"
            SELECT * FROM shares WHERE recipient_id = $1
            "#,
        )
        .bind(recipient_id)
        .fetch_all(pool)
        .await
        .map_err(AppError::from)?;

        Ok(shares)
    }

    pub async fn delete(pool: &PgPool, id: Uuid, user_id: Uuid) -> Result<(), AppError> {
        let share = Self::find_by_id(pool, id).await?

        if share.owner_id != user_id {
            return Err(AppError::AccessDenied("You can only delete shares you own".to_string()));
        }

        sqlx::query(
            r#"
            DELETE FROM shares WHERE id = $1
            "#,
        )
        .bind(id)
        .execute(pool)
        .await
        .map_err(AppError::from)?;

        Ok(())
    }

    pub async fn get_full_share_details(pool: &PgPool, id: Uuid) -> Result<ShareResponse, AppError> {
        let share = Self::find_by_id(pool, id).await?

        let file = File::find_by_id(pool, share.file_id).await?
        let owner = User::find_by_id(pool, share.owner_id).await?
        
        let recipient = match share.recipient_id {
            Some(recipient_id) => Some(User::find_by_id(pool, recipient_id).await?),
            None => None,
        };

        let permission = Permission::find_by_id(pool, share.permission_id).await?

        Ok(ShareResponse {
            id: share.id,
            file,
            owner,
            recipient,
            permission,
            access_key: share.access_key,
            expires_at: share.expires_at,
            created_at: share.created_at,
            updated_at: share.updated_at,
        })
    }

    pub async fn is_valid_share(
        pool: &PgPool,
        share_id: Uuid,
        user_id: Option<Uuid>,
    ) -> Result<bool, AppError> {
        let share = Self::find_by_id(pool, share_id).await?

        if let Some(expires_at) = share.expires_at {
            if expires_at < OffsetDateTime::now_utc() {
                return Ok(false);
            }
        }

        // If share has a specific recipient, check if the user is that recipient - otherwise return a valid share for everyone with the link
        if let Some(recipient_id) = share.recipient_id {
            if let Some(user_id) = user_id {
                return Ok(recipient_id == user_id);
            }
            return Ok(false);
        }
        Ok(true)
    }
}