use argon2::{password_hash::SaltString, Argon2, PasswordHash, PasswordHasher, PasswordVerifier};
use rand::rngs::OsRng;
use serde::{Deserialize, Serialize};
use sqlx::{postgres::PgPool, FromRow};
use time::OffsetDateTime;
use uuid::Uuid;

use crate::error::AppError;

#[derive(Debug, Serialize, Deserialize, FromRow)]
pub struct User {
    pub id: Uuid,
    pub email: String,
    #[serde(skip_serializing)]
    pub password_hash: String,
    pub display_name: Option<String>,
    pub storage_used: i64,
    pub storage_quota: i64,
    pub created_at: OffsetDateTime,
    pub updated_at: OffsetDateTime,
}

#[derive(Debug, Deserialize)]
pub struct CreateUserDto {
    pub email: String,
    pub password: String,
    pub display_name: Option<String>,
}

#[derive(Debug, Deserialize)]
pub struct LoginUserDto {
    pub email: String,
    pub password: String,
}

impl User {
    pub async fn create(pool: &PgPool, dto: CreateUserDto) -> Result<Self, AppError> {
        let existing_user = sqlx::query!("SELECT id FROM users WHERE email = $1", dto.email)
            .fetch_optional(pool)
            .await?;

        if existing_user.is_some() {
            return Err(AppError::UserAlreadyExists);
        }
        let password_hash = Self::hash_password(&dto.password)?;
        
        let storage_quota = 1_073_741_824;

        let user = sqlx::query_as!(User,
            r#"INSERT INTO users (id, email, password_hash, display_name, storage_used, storage_quota, created_at, updated_at)
            VALUES ($1, $2, $3, $4, $5, $6, $7, $8)
            RETURNING id, email, password_hash, display_name, storage_used, storage_quota, created_at, updated_at"#,
            Uuid::new_v4(),
            dto.email,
            password_hash,
            dto.display_name,
            0i64,
            storage_quota,
            OffsetDateTime::now_utc(),
            OffsetDateTime::now_utc()
        )
        .fetch_one(pool)
        .await?;

        Ok(user)
    }

    pub async fn find_by_id(pool: &PgPool, id: Uuid) -> Result<Self, AppError> {
        let user = sqlx::query_as!(User,
            r#"SELECT id, email, password_hash, display_name, storage_used, storage_quota, created_at, updated_at
            FROM users WHERE id = $1"#,
            id
        )
        .fetch_optional(pool)
        .await?
        .ok_or(AppError::UserNotFound)?;

        Ok(user)
    }

    pub async fn find_by_email(pool: &PgPool, email: &str) -> Result<Self, AppError> {
        let user = sqlx::query_as!(User,
            r#"SELECT id, email, password_hash, display_name, storage_used, storage_quota, created_at, updated_at
            FROM users WHERE email = $1"#,
            email
        )
        .fetch_optional(pool)
        .await?
        .ok_or(AppError::UserNotFound)?;

        Ok(user)
    }

    pub async fn authenticate(pool: &PgPool, dto: LoginUserDto) -> Result<Self, AppError> {
        let user = Self::find_by_email(pool, &dto.email).await?;

        if !Self::verify_password(&dto.password, &user.password_hash)? {
            return Err(AppError::InvalidCredentials);
        }

        Ok(user)
    }

    pub async fn update_storage_used(pool: &PgPool, user_id: Uuid, bytes_added: i64) -> Result<(), AppError> {
        let mut user = Self::find_by_id(pool, user_id).await?;
        
        let new_storage_used = user.storage_used + bytes_added;
        
        if new_storage_used > user.storage_quota {
            return Err(AppError::StorageQuotaExceeded);
        }
        sqlx::query!("UPDATE users SET storage_used = $1, updated_at = $2 WHERE id = $3",
            new_storage_used,
            OffsetDateTime::now_utc(),
            user_id
        )
        .execute(pool)
        .await?;
        
        Ok(())
    }

    fn hash_password(password: &str) -> Result<String, AppError> {
        let salt = SaltString::generate(&mut OsRng);
        let argon2 = Argon2::default();
        let password_hash = argon2
            .hash_password(password.as_bytes(), &salt)?
            .to_string();
        Ok(password_hash)
    }

    fn verify_password(password: &str, hash: &str) -> Result<bool, AppError> {
        let parsed_hash = PasswordHash::new(hash)?;
        let argon2 = Argon2::default();
        Ok(argon2.verify_password(password.as_bytes(), &parsed_hash).is_ok())
    }
}