use axum::{
    extract::Extension,
    routing::post,
    Json, Router,
};
use sqlx::PgPool;
use std::sync::Arc;

use crate::config::Config;
use crate::error::AppError;
use crate::models::user::{CreateUserDto, LoginUserDto, User};
use crate::utils::jwt;

pub fn routes() -> Router {
    Router::new()
        .route("/auth/register", post(register))
        .route("/auth/login", post(login))
}

async fn register(
    Extension(pool): Extension<PgPool>,
    Extension(config): Extension<Arc<Config>>,
    Json(create_user_dto): Json<CreateUserDto>,
) -> Result<Json<serde_json::Value>, AppError> {
    let user = User::create(&pool, create_user_dto, &config).await?;

    let token = jwt::generate_token(
        &config,
        user.id,
        &user.email,
        &user.role,
    )?;

    Ok(Json(serde_json::json!({
        "user": user,
        "token": token
    })))
}

async fn login(
    Extension(pool): Extension<PgPool>,
    Extension(config): Extension<Arc<Config>>,
    Json(login_dto): Json<LoginUserDto>,
) -> Result<Json<serde_json::Value>, AppError> {
    let user = User::authenticate(&pool, &login_dto.email, &login_dto.password).await?;

    let token = jwt::generate_token(
        &config,
        user.id,
        &user.email,
        &user.role,
    )?;

    Ok(Json(serde_json::json!({
        "user": user,
        "token": token
    })))
}