use axum::{
    extract::Extension,
    http::StatusCode,
    routing::{get, put},
    Json, Router,
};
use serde::{Deserialize, Serialize};
use sqlx::PgPool;

use crate::error::AppError;
use crate::models::user::User;
use crate::services::auth::AuthUser;

pub fn routes() -> Router {
    Router::new()
        .route("/users/me", get(get_current_user))
        .route("/users/me/password", put(update_password))
}

async fn get_current_user(
    auth_user: AuthUser,
    Extension(pool): Extension<PgPool>,
) -> Result<Json<User>, AppError> {
    let user = User::find_by_id(&pool, auth_user.id).await?;
    Ok(Json(user))
}

#[derive(Debug, Serialize, Deserialize)]
pub struct UpdatePasswordDto {
    pub current_password: String,
    pub new_password: String,
}

async fn update_password(
    auth_user: AuthUser,
    Extension(pool): Extension<PgPool>,
    Json(update_dto): Json<UpdatePasswordDto>,
) -> Result<StatusCode, AppError> {
    // Verify current password
    let user = User::find_by_id(&pool, auth_user.id).await?;
    let is_valid = crate::utils::password::verify_password(
        &update_dto.current_password,
        &user.password_hash,
    )?;

    if !is_valid {
        return Err(AppError::AuthenticationError("Current password is incorrect".to_string()));
    }

    // Hash new password
    let new_password_hash = crate::utils::password::hash_password(&update_dto.new_password)?;

    // Update password in database
    sqlx::query!(
        r#"
        UPDATE users
        SET password_hash = $1, updated_at = NOW()
        WHERE id = $2
        "#,
        new_password_hash,
        auth_user.id
    )
    .execute(&pool)
    .await
    .map_err(AppError::from)?;

    Ok(StatusCode::OK)
}