use axum::{
    async_trait,
    extract::{FromRequest, RequestParts, TypedHeader},
    headers::{authorization::Bearer, Authorization},
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use serde::{Deserialize, Serialize};
use std::sync::Arc;
use uuid::Uuid;

use crate::config::Config;
use crate::error::AppError;
use crate::utils::jwt;

#[derive(Debug, Serialize, Deserialize)]
pub struct AuthUser {
    pub id: Uuid,
    pub email: String,
    pub role: String,
}

#[async_trait]
impl<B> FromRequest<B> for AuthUser
where
    B: Send,
{
    type Rejection = Response;

    async fn from_request(req: &mut RequestParts<B>) -> Result<Self, Self::Rejection> {
        let TypedHeader(Authorization(bearer)) =
            TypedHeader::<Authorization<Bearer>>::from_request(req)
                .await
                .map_err(|_| {
                    let json = Json(serde_json::json!({
                        "error": "Missing or invalid authorization header"
                    }));
                    (StatusCode::UNAUTHORIZED, json).into_response()
                })?;

        let config = req
            .extensions()
            .get::<Arc<Config>>()
            .ok_or_else(|| {
                let json = Json(serde_json::json!({
                    "error": "Server configuration error"
                }));
                (StatusCode::INTERNAL_SERVER_ERROR, json).into_response()
            })?;

        let claims = jwt::verify_token(config, bearer.token()).map_err(|e| {
            let json = Json(serde_json::json!({
                "error": format!("Invalid token: {}", e)
            }));
            (StatusCode::UNAUTHORIZED, json).into_response()
        })?;

        let user_id = Uuid::parse_str(&claims.sub).map_err(|_| {
            let json = Json(serde_json::json!({
                "error": "Invalid user ID in token"
            }));
            (StatusCode::UNAUTHORIZED, json).into_response()
        })?;

        Ok(AuthUser {
            id: user_id,
            email: claims.email,
            role: claims.role,
        })
    }
}