use aes_gcm::{aead::{Aead, KeyInit}, Aes256Gcm, Nonce};
use base64::{engine::general_purpose, Engine as _};
use rand::{rngs::OsRng, RngCore};

use crate::error::AppError;

pub struct EncryptionService {
    cipher: Aes256Gcm,
}

impl EncryptionService {
    pub fn new(master_key: &str) -> Self {
        // Decode the base64 master key
        let key_bytes = general_purpose::STANDARD
            .decode(master_key)
            .expect("Invalid master key format");

        // Create the cipher
        let cipher = Aes256Gcm::new_from_slice(&key_bytes)
            .expect("Invalid key length");

        Self { cipher }
    }

    pub fn encrypt(&self, data: &[u8]) -> Result<Vec<u8>, AppError> {
        // Generate a random 12-byte nonce
        let mut nonce_bytes = [0u8; 12];
        OsRng.fill_bytes(&mut nonce_bytes);
        let nonce = Nonce::from_slice(&nonce_bytes);

        // Encrypt the data
        let ciphertext = self.cipher
            .encrypt(nonce, data)
            .map_err(|e| AppError::EncryptionError(e.to_string()))?;

        // Combine nonce and ciphertext
        let mut result = Vec::with_capacity(nonce_bytes.len() + ciphertext.len());
        result.extend_from_slice(&nonce_bytes);
        result.extend_from_slice(&ciphertext);

        Ok(result)
    }

    pub fn decrypt(&self, data: &[u8]) -> Result<Vec<u8>, AppError> {
        // Extract nonce and ciphertext
        if data.len() < 12 {
            return Err(AppError::EncryptionError("Invalid encrypted data format".to_string()));
        }

        let nonce = Nonce::from_slice(&data[..12]);
        let ciphertext = &data[12..];

        // Decrypt the data
        let plaintext = self.cipher
            .decrypt(nonce, ciphertext)
            .map_err(|e| AppError::EncryptionError(e.to_string()))?;

        Ok(plaintext)
    }

    pub fn generate_master_key() -> String {
        // Generate a random 32-byte key
        let mut key = [0u8; 32];
        OsRng.fill_bytes(&mut key);

        // Encode as base64
        general_purpose::STANDARD.encode(key)
    }
}