use std::path::{Path, PathBuf};
use tokio::fs;
use uuid::Uuid;

use crate::error::AppError;

pub struct StorageService {
    base_path: PathBuf,
}

impl StorageService {
    pub fn new<P: AsRef<Path>>(base_path: P) -> Self {
        Self {
            base_path: PathBuf::from(base_path.as_ref()),
        }
    }

    pub async fn save_file(&self, user_id: Uuid, file_id: Uuid, data: &[u8]) -> Result<(), AppError> {
        let file_path = self.get_file_path(user_id, file_id);
        
        // Ensure the directory exists
        if let Some(parent) = file_path.parent() {
            fs::create_dir_all(parent).await
                .map_err(|e| AppError::IoError(e.to_string()))?;
        }
        
        // Write the file
        fs::write(&file_path, data).await
            .map_err(|e| AppError::IoError(e.to_string()))?;
            
        Ok(())
    }
    
    pub async fn read_file(&self, user_id: Uuid, file_id: Uuid) -> Result<Vec<u8>, AppError> {
        let file_path = self.get_file_path(user_id, file_id);
        
        // Read the file
        let data = fs::read(&file_path).await
            .map_err(|e| AppError::IoError(e.to_string()))?;
            
        Ok(data)
    }
    
    pub async fn delete_file(&self, user_id: Uuid, file_id: Uuid) -> Result<(), AppError> {
        let file_path = self.get_file_path(user_id, file_id);
        
        // Check if file exists
        if !file_path.exists() {
            return Ok(());
        }
        
        // Delete the file
        fs::remove_file(&file_path).await
            .map_err(|e| AppError::IoError(e.to_string()))?;
            
        Ok(())
    }
    
    pub async fn create_user_directory(&self, user_id: Uuid) -> Result<(), AppError> {
        let dir_path = self.get_user_directory(user_id);
        
        // Create the directory if it doesn't exist
        if !dir_path.exists() {
            fs::create_dir_all(&dir_path).await
                .map_err(|e| AppError::IoError(e.to_string()))?;
        }
        
        Ok(())
    }
    
    pub fn get_file_path(&self, user_id: Uuid, file_id: Uuid) -> PathBuf {
        let mut path = self.get_user_directory(user_id);
        path.push(file_id.to_string());
        path
    }
    
    pub fn get_user_directory(&self, user_id: Uuid) -> PathBuf {
        let mut path = self.base_path.clone();
        path.push(user_id.to_string());
        path
    }
}