use crate::config::Config;
use crate::error::AppError;
use jsonwebtoken::{decode, encode, DecodingKey, EncodingKey, Header, Validation};
use serde::{Deserialize, Serialize};
use std::sync::Arc;
use time::{Duration, OffsetDateTime};
use uuid::Uuid;

#[derive(Debug, Serialize, Deserialize)]
pub struct Claims {
    pub sub: String,
    pub exp: i64,
    pub iat: i64,
    pub email: String,
    pub role: String, 
}

pub fn generate_token(
    config: &Arc<Config>,
    user_id: Uuid,
    email: &str,
    role: &str,
) -> Result<String, AppError> {
    let now = OffsetDateTime::now_utc();
    let expires_at = now + Duration::seconds(config.jwt_expiration);

    let claims = Claims {
        sub: user_id.to_string(),
        exp: expires_at.unix_timestamp(),
        iat: now.unix_timestamp(),
        email: email.to_string(),
        role: role.to_string(),
    };

    let token = encode(
        &Header::default(),
        &claims,
        &EncodingKey::from_secret(config.jwt_secret.as_bytes()),
    )
    .map_err(|e| AppError::JwtError(e.to_string()))?;

    Ok(token)
}

pub fn verify_token(config: &Arc<Config>, token: &str) -> Result<Claims, AppError> {
    let token_data = decode::<Claims>(
        token,
        &DecodingKey::from_secret(config.jwt_secret.as_bytes()),
        &Validation::default(),
    )
    .map_err(|e| AppError::JwtError(e.to_string()))?;

    Ok(token_data.claims)
}